/***
 * Memory bitmap
 *
 * Change Log:
 * 2003/11/20 ghort	created
 */


#ifndef _BITMAP_H_
#define _BITMAP_H_

#include "../../assignment-2.h"
#include "defs.h"


/*
 * Bitmap of allocated blocks. This bitmap is shared among threads with the
 * same mapping.
 */
typedef struct mem_bitmap_s {
  struct mutex mtx;           /* Mutex for protecting data in this structure */
  unsigned char *map;    /* bitmap of allocated 4KB blocks */
  int size;              /* size of the bitmap */
  int alloc_count;       /* number of allocated blocks */
} mem_bitmap_s;

/* Initializes a bitmap of memory blocks */
void bitmap_init(mem_bitmap_s *bmp, int size);

/* Locks the bmp structure */
void bitmap_lock(mem_bitmap_s *bmp);

/* Unlocks the bmp structure */
void bitmap_unlock(mem_bitmap_s *bmp);


/* All the following functions must be called with locked bmp structure */
/* vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv */

/* Prints content of the bitmap. Prints at most _len_ cells. */
void bitmap_print(mem_bitmap_s *bmp, int len);

/* Returns count of free blocks in bitmap. */
int block_free_count(mem_bitmap_s *bmp);

/* Returns count of allocated blocks in bitmap. */
int block_alloc_count(mem_bitmap_s *bmp);

/* Returns number of free pages starting at start. Looks for at most max pages. */
int block_count_free(mem_bitmap_s *bmp, int start, int max);

/* Returns number of full pages starting at start. Looks for at most max pages. */
int block_count_alloc(mem_bitmap_s *bmp, int start, int max);

/* This function tests whether a block starting at start does overlap with another blocks previously allocated by us.
 * Start and size are counts of 4KB blocks. Returns 0 if the block does not overlap. */
int block_overlap(mem_bitmap_s *bmp, int start, int size);

/* Finds nth free block in bitmap starting from 0. Returns -1 if no free block is available. */
int block_find_free(mem_bitmap_s *bmp, int nth);

/* Finds nth full block in bitmap starting from 0. Returns -1 if no full block is available. */
int block_find_full(mem_bitmap_s *bmp, int nth);

/* Marks blocks in the bitmap full. Start and size are counts of 4KB blocks. */
void block_mark_full(mem_bitmap_s *bmp, int start, int size);

/* Marks blocks in the bitmap free. Start and size are counts of 4KB blocks. */
void block_mark_free(mem_bitmap_s *bmp, int start, int size);

/* Returns -1 if the blocks contains more than cnt sequences of 0 or 1. */
int block_sequence_max(mem_bitmap_s *bmp, int start, int size, int starts_with, int seqcnt);

#endif /* _BITMAP_H_ */

